package ca.concordia.jdeodorant.eclipse.commandline.cloneinfowriter;

import gr.uom.java.ast.decomposition.cfg.mapping.precondition.PreconditionViolation;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class CloneInfoCSVWriter extends CloneInfoWriter {
	private final static String GLOBAL_CSV_FILE_NAME = "report.csv";
	private final static String PRECOND_VIOLATIONS_FILE_NAME = "precondviolations.csv";
	private static final String MAPPERS_FILE_NAME = "trees.csv";
	private final List<String> mainCSVLines = new ArrayList<>();
	private final List<String> mappersCSVLines = new ArrayList<>();
	private final List<String> preconditionViolationsLines = new ArrayList<>();
	public static final String PATH_TO_CSV_FILES = "";

	public CloneInfoCSVWriter(String outputFolder, String projectName, String fileNamePrefix) {
		super(outputFolder + "/" + PATH_TO_CSV_FILES, projectName, fileNamePrefix);
		createHeadersForFiles();
	}

	private void createHeadersForFiles() {
		mainCSVLines.add("GroupID|PairID|ClonePairLocation|IsTestCode|" +
				"#StatementsInCloneFragment1|#StatementsInCloneFragment2|#NodeComparisons|#PDGNodesInMethod1|#PDGNodesInMethod2|" +
				"#RefactorableSubtrees|SubtreeMatchingWallNanoTime|Status");
		mappersCSVLines.add("GroupID|PairID|TreeID|CloneType|PDGMappingWallNanoTime|#PreconditionViolations|#MappedStatements|#UnMappedStatements1|#UnMappedStatements2|#Differences");
		preconditionViolationsLines.add("GroupID|PairID|TreeID|PreconditionViolationType");
	}

	@Override
	public void writeCloneInfo(ClonePairInfo pairInfo) {
		
		StringBuilder line = new StringBuilder();
		
		line.append(pairInfo.getCloneGroupID()).append("|");
		line.append(pairInfo.getClonePairID()).append("|");
		line.append(pairInfo.getLocation().ordinal()).append("|");
		line.append(pairInfo.pairTestCodeInfo().ordinal()).append("|");
		line.append(pairInfo.getNumberOfCloneStatementsInFirstCodeFragment()).append("|");
		line.append(pairInfo.getNumberOfCloneStatementsInSecondCodeFragment()).append("|");
		line.append(pairInfo.getNumberOfNodeComparisons()).append("|");
		line.append(pairInfo.getNumberOfPDGNodesInFirstMethod()).append("|");
		line.append(pairInfo.getNumberOfPDGNodesInSecondMethod()).append("|");
		line.append(pairInfo.getRefactorableMappersInfo().size()).append("|");
		line.append(pairInfo.getSubtreeMatchingWallNanoTime()).append("|");
		line.append(pairInfo.getStatus().ordinal());
		mainCSVLines.add(line.toString());
		
		int treeID = 0;
		for (PDGSubTreeMapperInfo mapperInfo : pairInfo.getPDFSubTreeMappersInfoList()) {
			treeID++;
			line = new StringBuilder();
			line.append(pairInfo.getCloneGroupID()).append("|");
			line.append(pairInfo.getClonePairID()).append("|");
			line.append(treeID).append("|");
			line.append(mapperInfo.getMapper().getCloneType().ordinal() + 1).append("|");
			line.append(mapperInfo.getWallNanoTimeElapsedForMapping()).append("|");
			line.append(mapperInfo.getMapper().getPreconditionViolations().size()).append("|");
			line.append(mapperInfo.getMapper().getRemovableNodesG1().size()).append("|");
			line.append(mapperInfo.getMapper().getRemainingNodesG1().size()).append("|");
			line.append(mapperInfo.getMapper().getRemainingNodesG2().size()).append("|");
			if (mapperInfo.getMapper().getMaximumStateWithMinimumDifferences() != null)
				line.append(mapperInfo.getMapper().getNodeDifferences().size());
			else 
				line.append("N/A");
			mappersCSVLines.add(line.toString());
			for (PreconditionViolation pv : mapperInfo.getMapper().getPreconditionViolations()) {
				line = new StringBuilder();
				line.append(pairInfo.getCloneGroupID()).append("|");
				line.append(pairInfo.getClonePairID()).append("|");
				line.append(treeID).append("|");
				line.append(pv.getType().ordinal());
				preconditionViolationsLines.add(line.toString());
			}
		}
	}
	
	/**
	 * Writes one line per each object in the given List.
	 * @param lines
	 * @param path
	 * @param append If set to true, we skip the first line in the List (the header line)
	 */
	public static void writeLinesToFile(List<?> lines, String path, boolean append) {
		try {

			BufferedWriter fw = openFile(path, append);

			for (int i = append ? 1 : 0; i < lines.size(); i++) {
				writeFile(fw, lines.get(i).toString());
			}

			closeFile(fw);

		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static BufferedWriter openFile(String path, boolean append) throws IOException {
		File f = new File(path);
		BufferedWriter fw = new BufferedWriter(new FileWriter(f, append));
		return fw;
	}

	public static void closeFile(BufferedWriter fw) throws IOException {
		fw.close();
	}

	public static void writeFile(BufferedWriter fw, String line) throws IOException {
		fw.append(line + "\r\n");
	}

	@Override
	public void closeMedia(boolean append) {
		writeLinesToFile(mainCSVLines, this.outputFileSaveFolder + outputFileNamesPrefix + "." + GLOBAL_CSV_FILE_NAME, append);
		writeLinesToFile(mappersCSVLines, this.outputFileSaveFolder + outputFileNamesPrefix + "." + MAPPERS_FILE_NAME, append);
		writeLinesToFile(preconditionViolationsLines, this.outputFileSaveFolder + outputFileNamesPrefix + "." + PRECOND_VIOLATIONS_FILE_NAME, append);
		
	}
	
	
}
